import argparse
import os
import sys
import unittest

sys.path.append(os.getcwd())

from paifulogger import log


ATTR = [
    "url",
    "lang",
    "format",
    "all-formats",
    "remake",
    "output",
    "version",
    "mjai",
    "ignore-duplicated",
]


class Test(unittest.TestCase):
    def test1(self):
        parser = argparse.ArgumentParser()
        args = parser.parse_args()

        for attr in ATTR:
            setattr(args, attr.replace("-", "_"), None)

        args.url = [
            # yonma tests
            "https://tenhou.net/3/?log=2022052501gm-00c1-0000-f71e7910&tw=1",
            "https://tenhou.net/0/?log=2023051123gm-0001-0000-b3720f99&tw=2",
            "https://tenhou.net/0/?log=2023051216gm-0001-0000-5fd022cc&tw=1",
            # sanma tests
            "https://tenhou.net/0/?log=2023050200gm-0099-0000-50c65fbf&tw=2",
            "https://tenhou.net/0/?log=2023050419gm-0099-0000-b3111c7e&tw=0",
        ]
        args.all_formats = True
        args.ignore_duplicated = True
        args.output = "./test/"

        self.assertRaises(Exception, log.log(args))


if __name__ == "__main__":
    unittest.main()



"""
URL Logger and Duplicate Checker

This module provides functionality to save URLs to a log file and check if a URL is duplicated in the log.

Log File:
    The module uses an HDF5 file format (url_log.h5) to store URLs. The log file is created automatically when
    you log the first URL.

Functions:
    - url_log(url, local_str: local_str, output: str) -> None:
        Logs the given URL to the log file.

    - check_duplicate(url, local_str: local_str, output: str) -> bool:
        Checks if the given URL already exists in the log file.

"""

from pandas import HDFStore, DataFrame
import os
from .i18n import local_str


def check_duplicate(url, local_str: local_str, output: str) -> bool:
    path = f"{output}/{local_str.paifu}"
    if not os.path.isdir(path):
        os.makedirs(path)
    store = HDFStore(f"{path}/url_log.h5")
    if "url" not in store:
        store["url"] = DataFrame(columns=["url"])
    duplicated = url.split("//")[1] in store["url"]["url"].values
    store.close()
    return duplicated


def url_log(url, local_str: local_str, output: str) -> None:
    path = f"{output}/{local_str.paifu}"
    if not os.path.isdir(path):
        os.makedirs(path)
    store = HDFStore(f"{path}/url_log.h5")
    if "url" not in store:
        store["url"] = DataFrame(columns=["url"])
    urls = store["url"]
    urls.loc[len(urls)] = url.split("//")[1]
    store["url"] = urls
    store.close()



import json
import os
import sys

bundle_dir = getattr(sys, '_MEIPASS', os.path.abspath(os.path.dirname(__file__)))


class local_str:
    def __init__(self, lang: str = "en", main_path: str = "./"):
        self.lang = lang
        self.main_path = main_path
        pass

    def load_data(self, data):
        for key in data:
            self.__setattr__(key, data[key])

    def __setattr__(self, name, val):
        super().__setattr__(name, val)

    def __getattr__(self, name):
        "If the attribute is not loaded, return the value in English."

        with open(f"{self.main_path}/localizations/en.json") as f:
            data = json.load(f)
        return data[name]


def localized_str(lang: str, main_path: str) -> local_str:
    localized = local_str(lang, main_path)
    try:
        with open(f"{main_path}/localizations/{lang}.json", encoding="utf-8") as f:
            data = json.load(f)
            localized.load_data(data)
    except FileNotFoundError:
        with open(f"{main_path}/localizations/en.json") as f:
            data = json.load(f)
            localized.load_data(data)
    return localized



import csv
from datetime import datetime
import re
import os.path
from pandas import read_csv

from .get_place import get_place
from .Paifu import Paifu
from .i18n import local_str


def log_into_csv(paifu: Paifu, local_str: local_str, output: str):
    if paifu.player_num == 3:
        paifu_str = local_str.sanma + local_str.paifu
    else:
        paifu_str = local_str.yonma + local_str.paifu
    path = f"{output}/{local_str.paifu}/{paifu_str}.csv"
    if os.path.isfile(path):
        csvfile = open(path, "a+", encoding="utf-8")
        writer = csv.writer(csvfile)
    else:
        csvfile = open(path, "w", encoding="utf-8")
        writer = csv.writer(csvfile)
        writer.writerow(
            [
                "# id",
                local_str.date,
                local_str.plc,
                local_str.paifu,
                local_str.preR,
            ]
        )
    df = read_csv(path, index_col=0, encoding="utf-8")
    writer.writerow(
        [
            df.shape[0],
            datetime.strptime(re.findall(r"\d{10}", paifu.url)[0], "%Y%m%d%H"),
            get_place(paifu, paifu.ban),
            paifu.url,
            float(paifu.r[paifu.ban]),
        ]
    )
    print(
        "csv: "
        + local_str.hint_record1
        + re.findall(r"\d{10}gm-\w{4}-\w{4}-\w{8}&tw=\d", paifu.url)[0]
        + local_str.hint_record2
    )
    csvfile.close()
    return None



import openpyxl as xl
from datetime import datetime
import re
from .get_place import get_place
from .Paifu import Paifu
from .i18n import local_str


def log_into_xlsx(paifu: Paifu, local_str: local_str, output: str):
    try:
        if paifu.player_num == 3:
            paifu_str = local_str.sanma + local_str.paifu
        else:
            paifu_str = local_str.yonma + local_str.paifu
        path = f"{output}/{local_str.paifu}/{paifu_str}.xlsx"
        wb = xl.load_workbook(path)
        sheet = wb.active
        sheet.delete_rows(sheet.max_row)
    except FileNotFoundError:
        wb = xl.Workbook()
        sheet = wb.active
        sheet.append(
            [
                local_str.date,
                local_str.plc,
                local_str.paifu,
                local_str.remark,
                local_str.preR,
            ]
        )
        sheet.column_dimensions["A"].width = 20
        sheet.column_dimensions["C"].width = 71
        sheet.column_dimensions["E"].width = local_str.excelE
    sheet.append(
        [
            datetime.strptime(re.findall(r"\d{10}", paifu.url)[0], "%Y%m%d%H"),
            get_place(paifu, paifu.ban),
            paifu.url,
            "",
            float(paifu.r[paifu.ban]),
        ]
    )
    sheet["C" + str(sheet.max_row)].style = "Hyperlink"
    sheet.append([local_str.avg_plc, "=average(B2:B" + str(sheet.max_row) + ")"])
    wb.save(path)
    print(
        "xlsx: "
        + local_str.hint_record1
        + re.findall(r"\d{10}gm-\w{4}-\w{4}-\w{8}&tw=\d", paifu.url)[0]
        + local_str.hint_record2
    )
    return None



import urllib.request
import gzip

HEADER = {
    "Host": "e.mjv.jp",
    "User-Agent": "Mozilla/5.0 (X11; Linux x86_64; rv:65.0) Gecko/20100101 Firefox/65.0",
    "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8",
    "Accept-Language": "en-US,en;q=0.5",
    "Accept-Encoding": "gzip, deflate",
    "Connection": "keep-alive",
}


def url_request_handler(url: str):
    url = url.split("=")[1]
    url = "https://tenhou.net/0/log/?" + url[:-3]
    req = urllib.request.Request(url=url, headers=HEADER)
    opener = urllib.request.build_opener()
    response = opener.open(req)
    response = gzip.decompress(response.read()).decode("utf-8")
    return response



import xml.etree.ElementTree as ET


class Paifu:
    def __init__(self, url: str, root: ET.Element):
        self.url = url
        self.ban = int(url[-1])
        self.root = root

        self.go_type = int(root[1].get("type"))
        self.owari = root[-1].get("owari").split(",")
        self.r = root[2].get("rate").split(",")

        self.go_type_distinguish()

    def go_type_distinguish(self):
        """
        Distinguish the type of the game. And set the go_str and player_num.

        ---

        Examples:
            三鳳南喰赤速 (go_type = 127) -> go_str = '三南喰赤速', player_num = 3

        """

        self.go_str = ""
        if self.go_type & 1:
            # PVP
            pass

        if self.go_type & 16:
            self.go_str += "三"
            self.player_num = 3
        else:
            self.go_str += "四"
            self.player_num = 4

        if self.go_type & 128:
            # 上
            pass

        if self.go_type & 32:
            # 特 or 鳳
            pass

        if self.go_type & 8:
            self.go_str += "南"
        else:
            self.go_str += "東"

        if self.go_type & 4:
            self.go_str += "喰"

        if not self.go_type & 2:
            self.go_str += "赤"

        if self.go_type & 64:
            self.go_str += "速"
